 /*
  Copyright (c) 2003 by      William A. Gatliff
  All rights reserved.      bgat@billgatliff.com

  See the file COPYING for details.

  This file is provided "as-is", and without any express or implied
  warranties, including, without limitation, the implied warranties of
  merchantability and fitness for a particular purpose.

  The author welcomes feedback regarding this file.

  This stub was partially funded by Danfoss A/S.  Thanks, guys!

  **

  $Id: lpc.c,v 1.1.1.1 2007/06/05 20:06:16 bpisupat Exp $

   h8300-elf-gcc -g -ms -mint32 -O2 -nostartfiles -nodefaultlibs \
   -DCRT0 -DH8300_H8S2148EDK main.c gdb.c h8300.c h8300.S \
   -Wl,--script=h8300-h8s2148edk-stub.x; h8300-elf-objdump --syms \
   a.out | grep _start ; h8300-elf-objcopy -O srec a.out a.mot

*/
#include <lpc21xx/lpc210x.h>
#include <lpc21xx/dev_cntrl.h>
#include <lpc21xx/lpc_ioctl.h>
#include <stdio.h>
#include <errno.h>
#include "gdb.h"
#include "p2106.h"

// Arm Register Definitions
typedef enum {
  r0,r1,r2, r3 ,r4 ,r5 ,r6 ,r7,\
  r8,r9,r10,r11,r12,r13,r14,r15\ 
} register_id_E;

// Arm7 Register Definitions
typedef struct {
  long r0;
  long r1;
  long r2;
  long r3;
  long r4;
  long r5;
  long r6;
  long r7;
  long r8;
  long r9;
  long r10;
  long r11;
  long r12;
  long r13;  // sp - stack pointer  
  long r14;  // lr - link register
  long r15;  // pc - program counter
  long cpsr; // psr - program status register
} register_file_S;

register_file_S gdb_register_file;

#define ARM7_STEPI_OPCODE 0x0F000000UL /* SWI Instruction - "or" in the 24 bit immediate number to pass in */
typedef unsigned long arm7_instr_t; 
char arm7_stepping;
arm7_instr_t *arm7_stepped_addr; 
arm7_instr_t arm7_stepped_instr; 

int gdb_peek_register_file (int id, long *val) {
  int retval = sizeof(long);
  switch (id) {
  case r0: *val = gdb_register_file.r0; break;
  case r1: *val = gdb_register_file.r1; break;
  case r2: *val = gdb_register_file.r2; break;
  case r3: *val = gdb_register_file.r3; break;
  case r4: *val = gdb_register_file.r4; break;
  case r5: *val = gdb_register_file.r5; break;
  case r6: *val = gdb_register_file.r6; break;
  case r7: *val = gdb_register_file.r7; break;
  case r8: *val = gdb_register_file.r8; break;
  case r9: *val = gdb_register_file.r9; break;
  case r10: *val = gdb_register_file.r10; break;
  case r11: *val = gdb_register_file.r11; break;
  case r12: *val = gdb_register_file.r12; break;
  case r13: *val = gdb_register_file.r13; break;
  case r14: *val = gdb_register_file.r14; break;
  case r15: *val = gdb_register_file.r15; break;
  default: retval = 0;
  }
  return retval;
}

int gdb_poke_register_file (int id, long val) {
  int retval = sizeof(long);
  switch (id) {
  case r0: gdb_register_file.r0 = val; break;
  case r1: gdb_register_file.r1 = val; break;
  case r2: gdb_register_file.r2 = val; break;
  case r3: gdb_register_file.r3 = val; break;
  case r4: gdb_register_file.r4 = val; break;
  case r5: gdb_register_file.r5 = val; break;
  case r6: gdb_register_file.r6 = val; break;
  case r7: gdb_register_file.r7 = val; break;
  case r8: gdb_register_file.r8 = val; break;
  case r9: gdb_register_file.r9 = val; break;
  case r10: gdb_register_file.r10 = val; break;
  case r11: gdb_register_file.r11 = val; break;
  case r12: gdb_register_file.r12 = val; break;
  case r13: gdb_register_file.r13 = val; break;
  case r14: gdb_register_file.r14 = val; break;
  case r15: gdb_register_file.r15 = val; break;
  default: retval = 0;
  }
  return retval;
}

void arm7_handle_exception (int sigval) {
  gdb_monitor_onentry();
  if (arm7_stepping) {
    *arm7_stepped_addr = arm7_stepped_instr;
    arm7_stepping = 0;
  }
  gdb_handle_exception(sigval);
}

/* short h8300_instr_len (long addr) { */
/*   /\* H8S instructions aren't all the same length.  This table is */
/*      instruction length vs. the first byte of its opcode, with a few */
/*      exceptions marked by a -1 *\/ */
/*   static signed char const op2len[256] = { */
/*     /\* 0 *\/ */
/*     2, -1, 2, 2, 2, 2, 2, 2, */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* 1 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* 2 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* 3 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* 4 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* 5 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     4, 2, 4, 2, 4, 2, 4, 2,  */

/*     /\* 6 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, -1, -1, 2, 2, 4, 4, */

/*     /\* 7 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     8, 4, 6, 4, 4, 4, 4, 4,  */

/*     /\* 8 *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* 9 *\/  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* a *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* b *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* c *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* d *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* e *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */

/*     /\* f *\/ */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*     2, 2, 2, 2, 2, 2, 2, 2,  */
/*   }; */

/*   unsigned char *op = (unsigned char *)addr; */
/*   signed short len = op2len[*op]; */

/*   if (-1 == len) { */
/*     if (0x6aU == *op) { */
/*       switch (*(op + 1) & 0xf0) { */
/*       case 0x10 : len = 6; break; */
/*       case 0x30 : len = 8; break; */
/*       default: len = 2; break; /\* error *\/ */
/*       } */
/*     } */
/*     else if (0x6bU == *op) { */
/*       switch (*(op + 1) & 0xf0) { */
/*       case 0x30 : len = 4; break; */
/*       case 0xa0 : len = 6; break; */
/*       default: len = 2; break; /\* error *\/ */
/*       } */
/*     } */
/*     else if(0x01U == *op) { */
/*       if (0x6aU == *(op + 2) || 0x6bU == *(op + 2)) { */
/*         if (0x20U & *(op + 3)) */
/*           len = 8; */
/*         else */
/*           len = 6; */
/*       } */
/*       else */
/*         len = op2len[*(op + 2)] + 2; */
/*     } */
/*   } */
/*   return len; */
/* } */

/* int h8300_is_branch (h8300_instr_t op) { */
/*   if ((0x4000U == (op & 0xf000U)) */
/*       || (0x5000U == (op & 0xf000U))) { */
/*     /\* any 4x or 5x opcode is a branch of some kind, except for */
/*        mulxu (50 and 51) and divxu (52 and 53) *\/ */
/*     switch (op & 0xff00) { */
/*     case 0x5000U: */
/*     case 0x5100U: */
/*     case 0x5200U: */
/*     case 0x5300U: */
/*       return 0; */
/*     default: */
/*       return 1; */
/*     } */
/*   } */
/*   return 0; */
/* } */

/* #define H8300OP_RTE(op)   (0x5670 == (op)) */
/* #define H8300OP_RTS(op)   (0x5470 == (op)) */
/* #define H8300OP_BSR8(op)  (0x5500 == ((op) & 0xff00)) */
/* #define H8300OP_BSR16(op) (0x5c00 == (op)) */
/* #define H8300OP_JMPER(op) (0x5900 == ((op) & 0xff00)) */
/* #define H8300OP_JMP24(op) (0x5a00 == ((op) & 0xff00)) */
/* #define H8300OP_JMP8(op)  (0x5b00 == ((op) & 0xff00)) */
/* #define H8300OP_JSRER(op) (0x5d00 == ((op) & 0xff00)) */
/* #define H8300OP_JSR24(op) (0x5e00 == ((op) & 0xff00)) */
/* #define H8300OP_JSR8(op)  (0x5f00 == ((op) & 0xff00)) */
/* #define H8300OP_TRAPA(op) (0x5700 == ((op) & 0xff00)) */

/* #define H8300OP_BCC16(op) (0x5800 == ((op) & 0xff00)) */
/* #define H8300OP_BCC8(op)  (0x4000 == ((op) & 0xf000)) */

/* #define H8300OP_BCC_BT 0 */
/* #define H8300OP_BCC_BF 1 */
/* #define H8300OP_BCC_BHI 2 */
/* #define H8300OP_BCC_BLS 3 */
/* #define H8300OP_BCC_BHS 4 */
/* #define H8300OP_BCC_BLO 5 */
/* #define H8300OP_BCC_BNE 6 */
/* #define H8300OP_BCC_BEQ 7 */
/* #define H8300OP_BCC_BVC 8 */
/* #define H8300OP_BCC_BVS 9 */
/* #define H8300OP_BCC_BPL 0xa */
/* #define H8300OP_BCC_BMI 0xb */
/* #define H8300OP_BCC_BGE 0xc */
/* #define H8300OP_BCC_BLT 0xd */
/* #define H8300OP_BCC_BGT 0xe */
/* #define H8300OP_BCC_BLE 0xf */

/* #define H8300_CCR_C(ccr)  ((ccr) & 1) */
/* #define H8300_CCR_V(ccr)  ((ccr) & 2) */
/* #define H8300_CCR_Z(ccr)  ((ccr) & 4) */
/* #define H8300_CCR_N(ccr)  ((ccr) & 8) */
/* #define H8300_CCR_U(ccr)  ((ccr) & 0x10) */
/* #define H8300_CCR_H(ccr)  ((ccr) & 0x20) */
/* #define H8300_CCR_UI(ccr) ((ccr) & 0x40) */
/* #define H8300_CCR_I(ccr)  ((ccr) & 0x80) */


/* long h8300_branch_dest (long pc) { */
/*   h8300_instr_t op = *(h8300_instr_t *)pc; */
/*   signed int disp; */
/*   long dest = pc + h8300_instr_len(pc); */
/*   char ccr = gdb_register_file.ccr; */
/*   short r; */
  
/*   if (H8300OP_JMPER(op) || H8300OP_JSRER(op)) { */
/*     r = (op & 0x00f0) >> 8; */
/*     switch (r) { */
/*     case 0: dest = gdb_register_file.r0; break; */
/*     case 1: dest = gdb_register_file.r1; break; */
/*     case 2: dest = gdb_register_file.r2; break; */
/*     case 3: dest = gdb_register_file.r3; break; */
/*     case 4: dest = gdb_register_file.r4; break; */
/*     case 5: dest = gdb_register_file.r5; break; */
/*     case 6: dest = gdb_register_file.r6; break; */
/*     case 7: dest = gdb_register_file.sp; break; */
/*     default: dest = 0; break; */
/*     } */
/*   } */
/*   else if (H8300OP_JMP24(op) || H8300OP_JSR24(op)) */
/*       dest = *(unsigned long*)pc & 0xffffffUL; */
/*   else if (H8300OP_JMP8(op) || H8300OP_JSR8(op) || H8300OP_BSR8(op)) */
/*       dest = op & 0x00ff; */
/*   else if (H8300OP_BSR16(op)) */
/*       dest = pc + *(signed short*)(pc + 2) + 2; */
/*   else if (H8300OP_TRAPA(op)) { */
/*     r = (op & 0x00f0) >> 8; */
/*     dest = ((unsigned long *)(0x20))[r]; */
/*   } */
/*   else if (H8300OP_RTE(op) || H8300OP_RTS(op)) */
/*       dest = (*(unsigned long*)gdb_register_file.sp) & 0xffffffUL; */
/*   else { */
/*     if (H8300OP_BCC8(op)) { */
/*       disp = (signed char)(op & 0xff); */
/*       r = (op & 0xf00) >> 8; */
/*     } */
/*     else /\* if (H8300OP_BCC16(op)) *\/ { */
/*       disp = *(signed short*)(pc + 2); */
/*       r = (op & 0xf0) >> 4; */
/*     } */
/*     switch (r) { */
/*     case H8300OP_BCC_BT: */
/*       dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BF: */
/*       break; */
/*     case H8300OP_BCC_BHI: */
/*       if (!(H8300_CCR_C(ccr) | H8300_CCR_Z(ccr))) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BLS: */
/*       if (H8300_CCR_C(ccr) | H8300_CCR_Z(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BHS: */
/*       if (!H8300_CCR_C(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BLO: */
/*       if (H8300_CCR_C(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BNE: */
/*       if (!H8300_CCR_Z(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BEQ: */
/*       if (H8300_CCR_Z(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BVC: */
/*       if (!H8300_CCR_V(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BVS: */
/*       if (H8300_CCR_V(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BPL: */
/*       if (!H8300_CCR_N(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BMI: */
/*       if (H8300_CCR_N(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BGE: */
/*       if (!(H8300_CCR_N(ccr) ^ H8300_CCR_V(ccr))) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BLT: */
/*       if (H8300_CCR_N(ccr) ^ H8300_CCR_V(ccr)) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BGT: */
/*       if (H8300_CCR_Z(ccr) | (!(H8300_CCR_N(ccr) ^ H8300_CCR_V(ccr)))) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     case H8300OP_BCC_BLE: */
/*       if (H8300_CCR_Z(ccr) | (H8300_CCR_N(ccr) ^ H8300_CCR_V(ccr))) */
/*         dest = pc + disp + 2; */
/*       break; */
/*     } */
/*   } */
/*   return dest; */
/* } */

/* void gdb_step (long addr) { */
/*   short len; */
/*   long dest; */
/*   h8300_instr_t op; */

/*   h8300_stepping = 1; */

/*   if (addr) */
/*     gdb_register_file.pc = addr; */

/*   op = *(h8300_instr_t*)gdb_register_file.pc; */
/*   len = h8300_instr_len(gdb_register_file.pc); */

/*   if (h8300_is_branch(op)) */
/*     dest = h8300_branch_dest(gdb_register_file.pc); */
/*   else */
/*     dest = gdb_register_file.pc + len; */
   
/*   h8300_stepped_addr = (h8300_instr_t *)dest; */
/*   h8300_stepped_instr = *h8300_stepped_addr; */

/*   *h8300_stepped_addr = H8300_STEPI_OPCODE; */
/*   gdb_return_from_exception(); */
/* } */


void gdb_continue (long addr) {
  //  gdb_return_from_exception();
}

void gdb_kill (void) {
  extern void start(void);
  start();
}

void gdb_detach (void) {
  extern void start(void);
  start();
}

void gdb_flush_cache (void *start, int len) {
  return;
}

const struct device_table_entry *device_table[] = {
  &com2,	/* stdin  */
  &com2,	/* stdout */
  &com2,	/* stderr */
  &com1,        /* com1 used by Bootloader and GDB stub */
  0
};

struct _reent *gdb_reent;
int gdb_chan;
extern void swi_interrupt_shell(void);

typedef struct SwiRegs 
{
  unsigned long r[16];     /* R0 - R12 */
} SwiRegs;

void swi_handler(unsigned long call_param,SwiRegs *r) {
  static volatile int count=0;
  // P2106_LED_ON;
  // iprintf("0x%02x swi_handler %d\n\r",call_param,count++);
  gdb_register_file.r0=r->r[0];
  gdb_register_file.r1=r->r[1];
  gdb_register_file.r2=r->r[2];
  gdb_register_file.r3=r->r[3];
  gdb_register_file.r4=r->r[4];
  gdb_register_file.r5=r->r[5];
  gdb_register_file.r6=r->r[6];
  gdb_register_file.r7=r->r[7];
  gdb_register_file.r8=r->r[8];
  gdb_register_file.r9=r->r[9];
  gdb_register_file.r10=r->r[10];
  gdb_register_file.r11=r->r[11];
  gdb_register_file.r12=r->r[12];
  gdb_register_file.r13=r->r[13];
  gdb_register_file.r14=r->r[14];
  gdb_register_file.r15=r->r[15];
  //  arm7_handle_exception (5);
}

void p2106_startup (void) {
  struct serial_param sp;

  IODIR |= (1<<LEDPIN);	// define LED-Pin as output
  IODIR &= ~(1<<SWPIN);	// define Switch-Pin as input
  P2106_LED_OFF;

  (void)SetNativeSpeed(14746uL);
  (void)SetMAM(3u,MAM_full_enable);
  (void)VPBControl(VPB_DIV1);
  (void)SetDesiredSpeed(60000uL);

  /* Desired serial line characteristics 115200,8n1 */
  sp.baud = 115200uL;
  sp.length = UART_WORD_LEN_8;
  sp.parity = UART_PARITY_NONE;
  sp.stop = UART_STOP_BITS_1;
  (void)ioctl(fileno(stdout),UART_SETUP,&sp);   // User Console 115200 8-N-1 (reference stdout, stdin, stderr) 
  gdb_chan=_open_r(gdb_reent,"com1",0,0);
  (void)ioctl(gdb_chan,UART_SETUP,&sp);         // GDB Debug Port 115200 8-N-1, device handle debug_chan
}

void gdb_monitor_onentry (void) {
  /* turn on USER led */
  P2106_LED_ON;
  iprintf("GDB Monitor Entry\n\r");
}

void gdb_monitor_onexit (void) {
  /* turn off USER led */
  P2106_LED_OFF;
  iprintf("GDB Monitor Exit\n\r");
}

void gdb_startup (void) {
  //  h8300_stepping = 0;
  //  stuff_regfile();
  p2106_startup();
}

int gdb_putc (int c) {
  _write_r(gdb_reent,gdb_chan,&c,1);
}

int gdb_getc (void) {
  char c;
  while (!(_read_r(gdb_reent,gdb_chan,&c,1)));
  return c;
}
