#include <stdio.h>
#include <string.h>
#include "jtag.h"
#include "target.h"
#include "arm7tdmi.h"
#include "error.h"
#include "dat.h"
#include "fns.h"
#include "styx.h"
#include "arch/thread.h"
//#include "LowLevelFunc.h"			// Low level user functions

#define N_REGS              16
#define PC_OFFSET 15  /* register number for PC */

/* max read size  */
#define MAXMEM 400

/* bounds for RAM*/
#define RAMSTART 0x40000000
#define RAMEND 0x40003FFF

/* escape chars for binary transfer  */
#define ESCAPE_BYTE 0x7d
#define ESCAPE_XOR_MASK 0x20

//extern register_file_S gdb_register_file;

u32 register_file[15];

extern Dev armdev;

enum{
  Qdir,
  Qregisters,
  Qmemory,
  Qstatus,
  Qcontrol
};

static const Dirtab
armtab[]={
  {".",    Qdir,   Qdir,  PDIR | 05},
  {"registers", Qregisters,  Qdir,  06},
  {"memory",    Qmemory, Qdir,  06},
  {"status",    Qstatus,  Qdir,  06},
  {"control",    Qcontrol, Qdir, 06}
};

extern Channel outChan;

Channel armChan = CHANNELN(Fcall *, 4);
Channel armfidChan = CHANNELN(iFid *, 4);


ErrorStruct armerrorstrt;

Thread armdevT;

static void
armread(iFid *c, void *buf, ushort n, vlong offset, ushort *len);

static void
armwrite(iFid *c, void *buf, ushort n, vlong offset, ushort *len);

static void
armwalk(iFid *c, char *name);

int char2hex(char c);
char hex2char(int h);

void
arminit(Dev *d);



/*
  This is the control loop for the arm device.
  It receives calls through the call pipe and dispatches them to the corresponding function.
*/

void armmain() {
  //  Fid *f;
  static Fcall *rx;
  static Fcall *tx;
  static iFid *f;
  char cmd;
  void *buf;
  u_int16_t count;
  u_int64_t offset;
  u_int8_t index;
  char *name;
  int status;
  u32 temp;
  
  printf("armmain: rcving on cpipe %p\n",&armChan);
  
  
  threadBegin();  
  
   
  
  while(1) {
    // receive call from the call pipe
    if (chanrecv(&armChan, &rx) < 0)
      threadReturn();
    printf("armthread: recvd 1  request\n");
    if (chanrecv(&armfidChan, &f) < 0)
      threadReturn();
    if (chanrecv(&armChan, &tx) < 0)
      threadReturn();

    if (!(setjmp(armerrorstrt.estack))) 
    switch(rx->type) {
 
    case Tread:
     count = rx->ns.un.Twrf.count;
     offset = rx->ns.un.Twrf.offset;
     armread(f, tx->ns.un.Rwrf.rdata + 1, count, offset, &(tx->ns.un.Rwrf.rcount));    
    break;
  
    case Twrite:
     count = rx->ns.un.Twrf.count;
     offset = rx->ns.un.Twrf.offset;
     buf = rx->ns.un.Twrf.data;
     armwrite(f, buf, count, offset, &(tx->ns.un.Rwrf.rcount));
    break;
  
    case Twalk:
     name = rx->ns.un.Twcf.name;
     armwalk(f, name);
     tx->ns.un.qid = f->qid;
    break;
  
    default:
     tx->type = Rerror;
     strcpy(tx->ename, "Invalid operation on armdev");
   }
   else
   {
     tx->type = Rerror;
     strcpy(tx->ename, armerrorstrt.errstr);
   }

   if (chansend(&outChan, &tx) < 0)
      threadReturn();    


    // call the corresponding function

    // send the response out of the output pipe

    // return the buffer
  }
  threadFinish();

}

static void
armread(iFid *f, void *vbuf, ushort n, vlong offset, ushort *len)
{
  char *tmp;
  int i; 
  char *buf;
 // char memory_read[8];
  u32 memory_read[80];
  u32 memval;
 // WORD memword[10];;
  int num32, num16, num8;
  long addr;
  u32 temp;
  
  int ret = 0;
  core_register_t *reg = 0;
  
  buf = (char *) vbuf;
 

  fprintf(stderr, "armread\n");
  
  if (f->qid.path & DMDIR)
    {
      dirread(f, (Dir *) vbuf, armtab, nelem(armtab), len, offset, &armerrorstrt);
      return;
    }
    
  switch(f->qid.path & 255) {
   
  case Qmemory:
  addr = offset;
  /* read buffer has to be aligned on a WORD boundary */  
   if ((addr > RAMEND) || (addr < RAMSTART)) {
    printf("WARNING: reading from illegal address %p\n", addr);
    bzero(buf, n);
    *len = n;
    return;
   }
  
   *len = n;
   
   if (addr & 1)   //not aligned on 2 word boundary
   {
     ret = arm7tdmi_memory_read8(buf++, addr++, 1);
     if (ret) {
        *len = 0;
        return;
     }
     len--;
   }
   if ((addr & 3) && (len > 1))   // not aligned on a 4 byte boundary 
   {
     ret = arm7tdmi_memory_read16((u16 *)memory_read, addr, 1);
     if (ret) {
        *len = 0;
        return;
     }
     memcpy(buf, memory_read, 2);
     addr += 2;
     n -= 2;
     buf += 2;
   }
   assert(!(addr & 3));
   assert(n >= 0);
 
   num32 = n/4; num16 = (n - 4 * num32)/2; num8 = n - 4*num32 - 2*num16;
   assert((num32 >=0) && (num16 >= 0) && (num8 >= 0));
 
   if (num32) {
    ret = arm7tdmi_memory_read32((u32 *)&(memory_read[0]), addr, num32);
//    ret = arm7tdmi_memory_read32(&memval, 0x40000314, 1); 
    if (ret) {
        *len = 0;
        return;
     }    
    memcpy(buf, memory_read, num32*4);
    buf += num32 * 4;
   }

   if (num16) {
    ret = arm7tdmi_memory_read16((u16 *)memory_read, addr + num32 * 4, num16);
    if (ret) {
        *len = 0;
        return;
     }
    memcpy(buf, memory_read, num16*2);
    buf += num16*2;
   }

   if (num8) {
    ret = arm7tdmi_memory_read8(memory_read, addr + num32 * 4 + num16 * 2, num8);
    if (ret) {
        *len = 0;
        return;
     }
    memcpy(buf, memory_read, num8);
    buf += num8;
   }  
    

  /*
  if  (MSP430_Read_Memory(offset, (BYTE*) buf, n) != STATUS_OK) {
     printf("Error: Unable to read memory\n");
     *len = 0;
     return;
   }
   */
 
 //status = MSP430_Read_Memory(0xE042, (BYTE*) memory_read, 8);
   
  
   return;
   break;
   
  case Qregisters :

    /***********************************************************************\
    In this case, n gives the number of registers to write (either 1 or all)
    In the case of n, the offset gives the register number 
    
    Note: we do not reread the PC since it has to be be handled with care ...
    \***********************************************************************/
 
   if (n == 8*N_REGS) {
     
   
   for(i=0;i<N_REGS;i++) {
     
     temp = register_file[i];
     
     /**************************************************************************************\
      The order in which data is sent out is: HNib of B1, LNib of B1, HNib of B0, LNib of B0

     \**************************************************************************************/
     
     buf[4*i] = hex2char((temp >> 28) & 0xF); 
     buf[4*i+1] = hex2char((temp >> 24) & 0xF); 
     buf[4*i+2] = hex2char((temp >> 20) & 0xF); 
     buf[4*i+3] = hex2char((temp >> 16) & 0xF); 
     buf[4*i+4] = hex2char((temp >> 12) & 0xF); 
     buf[4*i+5] = hex2char((temp >> 8) & 0xF); 
     buf[4*i+6] = hex2char((temp >> 4) & 0xF);
     buf[4*i+7] = hex2char(temp & 0xF);
   }
  
   *len = 8*N_REGS;   
   return;
   } else {
     if (n == 8) {
                            
      temp = register_file[offset];
   
      buf[0] = hex2char((temp >> 28) & 0xF); 
      buf[1] = hex2char((temp >> 24) & 0xF); 
      buf[2] = hex2char((temp >> 20) & 0xF); 
      buf[3] = hex2char((temp >> 16) & 0xF); 
      buf[4] = hex2char((temp >> 12) & 0xF); 
      buf[5] = hex2char((temp >> 8) & 0xF); 
      buf[6] = hex2char((temp >> 4) & 0xF);
      buf[7] = hex2char(temp & 0xF);
                 
       //   *n = 1;
       *len = 8;
       return;
     } else {
       printf("Error: Incorrect number of registers in write\n");
       *len = 0;
       return;
     }
   }
   break;  
   
  case Qstatus:
    //    sttime = nr_timer_milliseconds();
    if (offset>0) {
     *len = 0;
     return;
     break;
    }
    
   if (arm7tdmi_ishalted()) {
     buf[0] = 's';  
     arm7tdmi_read_registers_fromtarget();
   }
    else
     buf[0] = 'r';
  
   *len = 1;
   return;
   break; 
    
  default:
    printf("Read from invalid file.\n");
    *len = 0;
    return;
  }
   
}

static void
armwrite(iFid *f, void *vbuf, ushort n, vlong offset, ushort *len)
{
  u_int8_t *buf;
 
  int bkptno;
  u32 addr;
  u32 temp;
  int ret = 0; 
  int i; 
  u32 memory_buff[80];
  u8 *memory_write;
  int num32, num16, num8;
  u16 mem16write;
  u8 mem8write;
  u8 *memory_write_orig;
  
  memory_write = (u32 *) &memory_buff[0]; // buffer aligned on 4 bytes boundary
  memory_write_orig = memory_write;
    
  buf = (char *) vbuf;
  fprintf(stderr, "armwrite\n");
  
  if (f->qid.path & DMDIR)
   {
     *len = 0;
     return;
   }
  
   /* local file */

  switch(f->qid.path & 255) {
  case Qcontrol:
  
   switch (buf[0]) {
   case 's':   // stop device
     *len = n;
     if (n != 1) {
       printf("Error: Incorrect length for step command.");
       *len = 0;
       return;
     }

   
     return;
     break;
     
   case 't': //single step 
     /* incorporate error handling!    */
     *len = n; 
     if (n != 1) {
       printf("Error: Incorrect length for init command.\n");
       *len = 0;
       return;
     }

    arm7tdmi_write_registers_totarget();
    arm7tdmi_set_singlestep(register_file[15]);

     *len = 1;
     return;
     break;
  
  
   case 'c': //continue
     *len = n;
     if (n != 1) {
       printf("Error: Incorrect length for init command.\n");
       *len = 0;
       return;
     }

    arm7tdmi_write_registers_totarget();
    arm7tdmi_restart(register_file[PC_OFFSET]); //once + idelay

     
     return;
     break;

   case 'b': //breakpoint

     //     sttime = nr_timer_milliseconds();
     *len = n;
     bkptno = char2hex(buf[1]); // this is a number between '0' an '9' hopefully
     
    
     addr =  (char2hex(buf[2]) << 28) + (char2hex(buf[3]) << 24) + (char2hex(buf[4]) << 20) +\
               (char2hex(buf[5]) << 16) + (char2hex(buf[6]) << 12) + (char2hex(buf[7]) << 8) +\
                  (char2hex(buf[8]) << 4) + char2hex(buf[9]);
            
     if (addr) {
       if (arm7tdmi_ice_set_addrbreakpt(addr)) {
           printf("trouble setting breakpoint at addr\n");
           *len = 0;
           return;
       }
     } else
       if (arm7tdmi_ice_clear_breakpt()) {
           printf("trouble removing breakpoint at addr\n");
           *len = 0;
           return;
       }

     
     return;
     break;
     
    case 'e': //erase 

     *len = 0;
     return;
     break;


    default:
     printf("Warning: invalid operation : %c\n",buf[0]);
     *len = 0;
     return;
  }
     
 case Qmemory:
    addr = offset; 

    if ((addr > RAMEND) || (addr < RAMSTART)) {
    printf("WARNING: reading from illegal address %p\n", addr);
    bzero(buf, n);
    *len = n;
    return;
   }
   
   memcpy(memory_write, buf, n);
   *len = n;
   if (addr & 1)   //not aligned on 2 word boundary
   {
     mem8write = *memory_write++;
     ret = arm7tdmi_memory_write8((u8 *)&mem8write, addr++, 1);
     if (ret) {
        *len = 0;
        return;
     }
     n--;
   }
   if ((addr & 3) && (len > 1))   // not aligned on a 4 byte boundary 
   {
     mem16write = (u16)memory_write[0] | (((u16)((u8 *) memory_write)[1]) << 8);
     ret = arm7tdmi_memory_write16((u16 *) &mem16write, addr, 1);
     if (ret) {
        *len = 0;
        return;
     }
     addr += 2;     
     n -= 2;
     memory_write+=2;
   }
 
  assert(!(addr & 3));
  assert(n >= 0);

  num32 = n/4; num16 = (n - 4 * num32)/2; num8 = n - 4*num32 - 2*num16;
  assert((num32 >=0) && (num16 >= 0) && (num8 >= 0));
  
  /* shift the write array so as to be on a 32 bite boundary */
  if (memory_write_orig != memory_write)
    for(i = 0; i < n; i++)
      *((u8 *)((u8 *)memory_write_orig + i)) = *((u8 *)((u8 *)memory_write + i));
  
  memory_write = memory_write_orig;
  if (num32) {
    ret = arm7tdmi_memory_write32((u32 *) memory_write, addr, num32);
    if (ret) {
        *len = 0;
        return;
     }
    memory_write += num32 * 4;
  }

  if (num16) {
   ret = arm7tdmi_memory_write16((u16 *) memory_write, addr + num32 * 4, num16);
   if (ret) {
        *len = 0;
        return;
     }
   memory_write += num16 * 2;
  }

  if (num8) {
   ret = arm7tdmi_memory_write8((u8 *)memory_write, addr + num32 * 4 + num16 * 2, num8);
     if (ret) {
        *len = 0;
        return;
     }
  }
  

  return;
  break;
 
 /*
 case Qmemory:
    addr = offset; 

    if ((addr > RAMEND) || (addr < RAMSTART)) {
    printf("WARNING: reading from illegal address %p\n", addr);
    bzero(buf, n);
    *len = n;
    return;
   }
   
   memcpy(memory_write, buf, n);
   *len = n;
   if (addr & 1)   //not aligned on 2 word boundary
   {
     mem8write = memory_write++;
     ret = arm7tdmi_memory_write8((u8 *)memory_write++, addr++, 1);
     if (ret) {
        *len = 0;
        return;
     }
     n--;
   }
   if ((addr & 3) && (len > 1))   // not aligned on a 4 byte boundary 
   {
     ret = arm7tdmi_memory_write16((u16 *) memory_write, addr, 1);
     if (ret) {
        *len = 0;
        return;
     }
     addr += 2;     
     n -= 2;
     memory_write+=2;
   }
 
  assert(!(addr & 3));
  assert(n >= 0);

  num32 = n/4; num16 = (n - 4 * num32)/2; num8 = n - 4*num32 - 2*num16;
  assert((num32 >=0) && (num16 >= 0) && (num8 >= 0));
  
  if (num32) {
    ret = arm7tdmi_memory_write32((u32 *) memory_write, addr, num32);
    if (ret) {
        *len = 0;
        return;
     }
    memory_write += num32 * 4;
  }

  if (num16) {
   ret = arm7tdmi_memory_write16((u16 *) memory_write, addr + num32 * 4, num16);
   if (ret) {
        *len = 0;
        return;
     }
   memory_write += num16 * 2;
  }

  if (num8) {
   ret = arm7tdmi_memory_write8((u8 *)memory_write, addr + num32 * 4 + num16 * 2, num8);
     if (ret) {
        *len = 0;
        return;
     }
  }
  

  return;
  break;
  
  */
 
   
 case Qregisters:
 
   /************************************
    In this case, n gives the number of registers to write (either 1 or all)
    In the case of n, the offset gives the register number 
   *****************************************************/

   if (n == (8 * N_REGS)) { // 8 chars per register (2 chars per byte, 4 bytes)
     for(i=0;i<N_REGS;i++) {
       temp = (char2hex(buf[8*i+0]) << 28) + (char2hex(buf[8*i+1]) << 24) +\
               (char2hex(buf[8*i+2]) << 20) + (char2hex(buf[8*i+3]) << 16) +\
                (char2hex(buf[8*i+4]) << 12) + (char2hex(buf[8*i+5]) << 8) +\
                      (char2hex(buf[8*i+6]) << 4) + char2hex(buf[8*i+7]);
       register_file[i] = temp;
     }
     *len = 8 * N_REGS;
     return;
     }
   else {
     if (n == 8) {
       temp = (char2hex(buf[0]) << 28) + (char2hex(buf[1]) << 24) +\
               (char2hex(buf[2]) << 20) + (char2hex(buf[3]) << 16) +\
                (char2hex(buf[4]) << 12) + (char2hex(buf[5]) << 8) +\
                      (char2hex(buf[6]) << 4) + char2hex(buf[7]);
              
       register_file[offset] = temp;      
       *len = 8;
       return;
     } else {      
       printf("Error: Incorrect number of registers in write\n");
       *len = 0;
       return;
     }
   }
   printf("should never get here really\n");
   return;
   break; 
  
 default:
  printf("Warning: invalid qid %d\n",f->qid.path & 255);
  *len = 0;
  return;
  }

}
 

static void
armattach(iFid** f, char *spec)
{
  fprintf(stderr, "armattach\n");
  *f = newifid(-1, &armerrorstrt);
  (*f)->type = devno('A');
  (*f)->qid.path = (armtab[0].qid + (*f)->type * 256) | DMDIR;
}


static void
armwalk(iFid *f, char *name)
{
  fprintf(stderr, "armwalk\n");
  devwalk(f, name, armtab, nelem(armtab), &armerrorstrt);
}

void
armstat(iFid *f, Dir *d, ErrorStruct *es)
{
  fprintf(stderr, "armstat\n");
  devstat(f, d, &armtab[f->qid.path & 255], es);

}

static void
armopen(iFid **f, int omode, ErrorStruct *es)
{  
  fprintf(stderr, "armopen\n");
  devopen(f, armtab, omode,0, nelem(armtab), es);
}

void
arminit(Dev *d)
{
  u32 idcode;
  int ret = 0;
  core_register_t *reg = NULL;
    
  printf("Inside arminit.\n");
  armdevT.run = armmain;
  setThrdName(armdevT,'A');
  
  if (jtag_reset()) {
    printf("Error: failure to initialize jtag ...\n");
    exit(-1);
  }

  arm7tdmi_setup();
  jtag_idcode(&idcode);
 // printf("ARM device idcode = %x\n",idcode);
  printf("ARM device idcode = %d\n",idcode);
  ret = arm7tdmi_halt();
  if (ret)
    printf("halt: %s", err_str(ret)); 
  printf("halted successfully\n");
 
  _threadready(&armdevT);
  
  
}

void armclone(iFid **ppc, iFid *pc) {
  devclone(ppc, pc, &armerrorstrt);
}

void armcreate(iFid* pc, char* pch, int i, ulong ul) {
  devcreate(pc, pch, i, ul, &armerrorstrt);
}

void armclose(iFid *pc) {
  devclose(pc, &armerrorstrt);
}

void armremove(iFid* pc, ErrorStruct *es) {
   devremove(pc, es);
}

void armwstat(iFid* pc, Dir* pd, ErrorStruct *es) {
   devwstat(pc, pd, es);
}


Dev armdev = { 
  'A',
  "arm",
  &armChan,
  &armfidChan,
  arminit,
  armattach,
  armclone,
  armwalk,
  armstat,
  armopen,
  armcreate,
  armclose,
  armread,
  armwrite,
  armremove,
  armwstat
};


