/*========================================================================*\
|                                                                          |
| Device.c                                                                 |
|                                                                          |
| The data within this file is accessed via functions (i.e., the data is   |
| not global to the DLL).                                                  |
| This approach was taken to hopefully simplify the process of changing    |
| the DLL to use an .ini file for the data rather then a compiled file.    |
|                                                                          |
|--------------------------------------------------------------------------|
| Project:              MSP430 JTAG interfcae                              |
| Developed using:      MS Visual C++ 5.0                                  |
|--------------------------------------------------------------------------|
| Author:               Dale Wellborn                                      |
| Version:              1.14                                               |
| Initial Version:      20 / 07 / 02                                       |
| Last Change:                                                             |
|--------------------------------------------------------------------------|
| Version history:                                                         |
| Version: 1.0.14                                                          |
| 04/15/2003 UPSF Added FE427 and FW427                                    | 
|                 Reworked Device Descriptors and added MemModels          |
| 05/18/2003 UPSF Added F169                                               | 
| Version: 1.1.3                                                           |
| 16/07/2003 UPSF Added hasIC2                                             |
|                 Added separted F15x and F16x devices                     | 
| 29.07.2003 UPSF added Fuse Pattern for device Identification             |
| 08/13/2003 UPSF Added F1610 + F1611                                      | 
| Version: 1.1.4                                                           |
| 11/02/2003 UPSF Change Name of FW42x to FW42x/F41x                       | 
| 11/10/2003 UPSF Added new F169 (common ID with old one new)              | 
|                           FG437B                                         |
| 11/11/2003 UPSF Changed FG437 ID -> Dont care of Revision                | 
| Version: 1.1.5                                                           |
| 01/15/2004 UPSF                                                          |
|                 Changed Device Name: 'MSP430F12x2' -> 'MSP430F12x2/F11x2'| 
| 01/20/2004 UPSF Added MSP430F161x                                        |
| 03/12/2004 UPSF Added New MSP430FE42x                                    |
|                                                                          |
|--------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments                                       |
\*========================================================================*/

/*------------------------------------------------------------------------*\
| Remarks:                                                                 |
|                                                                          |
\*------------------------------------------------------------------------*/


// Device constants. NEVER change the enums below! Only add to the end of the data structures.
//

// #includes. -----------------------------------------------------------------

#include "MSP/Basic_Types.h"
#include "MSP/HIL.h"
#include "MSP/MSP430_Debug.h"

#include "MSP/Devices.h"
#include "MSP/Globals.h"

#ifdef _DEBUG
#include <stdio.h>
#endif

// File local variables. ------------------------------------------------------

// ****************************************************************************
// If devices and device information is added to the data structures in this file,
// the devices enumeration in MSP430.h must be extended as well.
//
// Device constants. NEVER change the enums below! Only add to the end of the 
// data structures.
//
// ****************************************************************************


int deviceTypeId = DT_UNKNOWN;       // Device identification for device Table
// Device Identification Parameters.
const struct tsdeviceID sdeviceID[] = 
{
   // Fuse parameter is set without lock bit
   //0x0   0x1   0x2   0x3   0x8   0x9   0xD   Fuse    Fuse    DeviceTypeID          Device String
   //ID0   ID1   Rev   Fab   Self0 Self1 Confg        Pattern
   {{ 0  ,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,    -1 },  DT_UNKNOWN,       "Device unknown"}, //Device unknown
   
   {{0xf1, 0x12, 0x00, 0x43, 0x00, 0x08,  -1 ,  -1 ,    -1 },  DT_MSP430F11x1,   "Prototype MSP430F11x1"},
   {{0xf1, 0x12, 0x10, 0x40, 0x00, 0x00,  -1 ,  -1 ,    -1 },  DT_MSP430F11x1D,  "MSP430F11x1"},

   {{0xf1, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x05 , 0x0f },  DT_MSP430F133,    "MSP430F133"},
   {{0xf1, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x04 , 0x0f },  DT_MSP430F135,    "MSP430F135"},
   {{0xf1, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x02 , 0x0f },  DT_MSP430F147,    "MSP430F147"},
   {{0xf1, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x01 , 0x0f },  DT_MSP430F148,    "MSP430F148"},
   {{0xf1, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x00 , 0x0f },  DT_MSP430F149,    "MSP430F149"},
   
   {{0xf4, 0x03, 0x00, 0x40,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F413P,   "Preliminary MSP430F413"},
   {{0xf4, 0x13, 0x02, 0x40,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F41xC,   "MSP430F41x"},
   
   {{0xf1, 0x12, 0x13, 0x40, 0x00, 0x00,  -1 ,  -1  ,   -1 },  DT_MSP430F11x1A,  "MSP430F11x1A"},
   
   {{0xf1, 0x23,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F12x,    "MSP430F12x"},
   
   {{0xf4, 0x37,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F43x,    "MSP430F43x"}, //- 80 Pin
   {{0xf4, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x02 , 0x0f },  DT_MSP430F43x,    "MSP430F43x"}, //- 100 Pin
   {{0xf4, 0x49,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x00 , 0x0f },  DT_MSP430F44x,    "MSP430F44x"}, 
   
   {{0x12, 0x32, 0x00, 0x40,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F12x2,   "Preliminary MSP430F12x2"},
   {{0x11, 0x32,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F11x2,   "MSP430F11x2"},
   {{0x12, 0x32, 0x10, 0x40,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_MSP430F12x2C,  "MSP430F12x2/F11x2"},
   {{0xf1, 0x69, 0x00, 0x40,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_PMS430F16x,    "Preliminary MSP430F16x"}, //DT_PMS430F169},
   
   {{0xf4, 0x27, 0x00,  -1 ,  -1 ,  -1 , 'E' , 0x02 , 0x1f },  DT_MSP430FE423,   "MSP430FE423_P"},
   {{0xf4, 0x27, 0x00,  -1 ,  -1 ,  -1 , 'E' , 0x01 , 0x1f },  DT_MSP430FE425,   "MSP430FE425_P"},
   {{0xf4, 0x27, 0x00,  -1 ,  -1 ,  -1 , 'E' , 0x00 , 0x1f },  DT_MSP430FE427,   "MSP430FE427_P"},
   {{0xf4, 0x27, 0x10,  -1 ,  -1 ,  -1 , 'E' , 0x02 , 0x1f },  DT_MSP430FE423,   "MSP430FE423_N"},
   {{0xf4, 0x27, 0x10,  -1 ,  -1 ,  -1 , 'E' , 0x01 , 0x1f },  DT_MSP430FE425,   "MSP430FE425_N"},
   {{0xf4, 0x27, 0x10,  -1 ,  -1 ,  -1 , 'E' , 0x00 , 0x1f },  DT_MSP430FE427,   "MSP430FE427_N"},

//   {{0xf4, 0x27, 0x20,  -1 ,  -1 ,  -1 , 'E' , 0x02 , 0x1f },  DT_MSP430FE423,   "MSP430FE423"}, // 2nd revision
//   {{0xf4, 0x27, 0x20,  -1 ,  -1 ,  -1 , 'E' , 0x01 , 0x1f },  DT_MSP430FE425,   "MSP430FE425"},
//   {{0xf4, 0x27, 0x20,  -1 ,  -1 ,  -1 , 'E' , 0x00 , 0x1f },  DT_MSP430FE427,   "MSP430FE427"},
//   {{0xf4, 0x27, 0x21,  -1 ,  -1 ,  -1 , 'E' , 0x02 , 0x1f },  DT_MSP430FE423,   "MSP430FE423"}, // 3nd revision
//   {{0xf4, 0x27, 0x21,  -1 ,  -1 ,  -1 , 'E' , 0x01 , 0x1f },  DT_MSP430FE425,   "MSP430FE425"},
//   {{0xf4, 0x27, 0x21,  -1 ,  -1 ,  -1 , 'E' , 0x00 , 0x1f },  DT_MSP430FE427,   "MSP430FE427"},
   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'E' , 0x02 , 0x1f },  DT_MSP430FE423,   "MSP430FE423"}, // 4nd revision
   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'E' , 0x01 , 0x1f },  DT_MSP430FE425,   "MSP430FE425"},
   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'E' , 0x00 , 0x1f },  DT_MSP430FE427,   "MSP430FE427"},

   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'E' , 0x06 , 0x1f },  DT_MSP430F423,    "MSP430F423"},  //F42x based on FE42x
   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'E' , 0x05 , 0x1f },  DT_MSP430F425,    "MSP430F425"},
   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'E' , 0x06 , 0x1f },  DT_MSP430F427,    "MSP430F427"},

//   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'W' ,  -1  ,   -1 },  DT_MSP430FW42x,   "MSP430FW42x"},
   {{0xf4, 0x27,  -1 ,  -1 ,  -1 ,  -1 , 'W' ,  -1  ,   -1 },  DT_MSP430FW42x,   "MSP430FW42x/F41x"},  // FW42x and F41x based on FW42x

   {{0xf1, 0x69,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x06 , 0x0f },  DT_MSP430F155,    "MSP430F155"},
   {{0xf1, 0x69,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x05 , 0x0f },  DT_MSP430F156,    "MSP430F156"},
   {{0xf1, 0x69,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x04 , 0x0f },  DT_MSP430F157,    "MSP430F157"},
   {{0xf1, 0x69,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x02 , 0x0f },  DT_MSP430F167,    "MSP430F167"},
   {{0xf1, 0x69,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x01 , 0x0f },  DT_MSP430F168,    "MSP430F168"},
   {{0xf1, 0x69,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x00 , 0x0f },  DT_MSP430F169,    "MSP430F169"},
   {{0xf1, 0x6C,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x03 , 0x07 },  DT_MSP430F1610,   "MSP430F1610"},
   {{0xf1, 0x6C,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x00 , 0x07 },  DT_MSP430F1611,   "MSP430F1611"},
   {{0xf1, 0x6C,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x06 , 0x07 },  DT_MSP430F1612,   "MSP430F1612"},
   {{0xf1, 0x6C,  -1 ,  -1 ,  -1 ,  -1 ,  -1 , 0x07 , 0x07 },  DT_MSP430F169,    "MSP430F169"},  //MSP430F169 based on F1611

   {{0xf4, 0x39,  -1 ,  -1 ,  -1 ,  -1 , 'G' ,  -1  ,   -1 },  DT_MSP430FG43x,   "MSP430FG43x"}, //DT_MSP430FG439

   {{ 0  ,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1 ,  -1  ,   -1 },  DT_END,           "End of devices"}, //End of devices
};

const WORD NUMofDevices = sizeof(sdeviceID)/sizeof(sdeviceID[0]);

struct tsMemModel {
     WORD  mainStart;
     ULONG mainEnd;
     WORD  infoStart;
     WORD  infoEnd;
     WORD  ramStart;
     WORD  ramEnd;
     WORD  ram2Start;
     WORD  ram2End;
   };

// Memory Models
enum
{
   MemModNo,       // Dummy for no Device
   MemMod_2P,      // First F1121 Device with Info at 0xEF00
   MemMod_0,       // Memory Model 0
   MemMod_1,       // Memory Model 1
   MemMod_2,       // Memory Model 2
   MemMod_3,       // Memory Model 3
   MemMod_4,       // Memory Model 4
   MemMod_5,       // Memory Model 5
   MemMod_6,       // Memory Model 6
   MemMod_7,       // Memory Model 7
   MemMod_8,       // Memory Model 8
   MemMod_9,       // Memory Model 9
   MemMod_10,      // Memory Model 10
   MemMod_11,      // Memory Model 11
   MemMod_12,      // Memory Model 12
};

// Device Memory Models Parameters.
const struct tsMemModel sMemModel[] = 
{
   //mainStart   mainEnd     infoStart   infoEnd   ramStart   ramEnd	    ram2Start   ram2End
   { 0      ,    0      ,    0      ,    0      ,    0      ,   0      ,    0      ,   0     },  // No Device
   { 0xf000 ,    0xffff ,    0xef00 ,    0xefff ,    0x200  ,   0x02ff ,    0      ,   0     },  // PMS430F1121 (Prototype)

   { 0xfc00 ,    0xffff ,    0x1080 ,    0x10ff ,    0x200  ,   0x027f ,    0      ,   0     },  // Memory Model 0
   { 0xf800 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x027f ,    0      ,   0     },  // Memory Model 1
   { 0xf000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x02ff ,    0      ,   0     },  // Memory Model 2
   { 0xe000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x02ff ,    0      ,   0     },  // Memory Model 3
   { 0xd000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x03ff ,    0      ,   0     },  // Memory Model 4
   { 0xc000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x03ff ,    0      ,   0     },  // Memory Model 5
   { 0xa000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x05ff ,    0      ,   0     },  // Memory Model 6
   { 0x8000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x05ff ,    0      ,   0     },  // Memory Model 7
   { 0x4000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x09ff ,    0      ,   0     },  // Memory Model 8
   { 0x1100 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x09ff ,    0      ,   0     },  // Memory Model 9
   { 0x8000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x09ff ,    0x1100 ,   0x24ff},  // Memory Model 10
   { 0x4000 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x09ff ,    0x1100 ,   0x38ff},  // Memory Model 11
   { 0x2500 ,    0xffff ,    0x1000 ,    0x10ff ,    0x200  ,   0x09ff ,    0x1100 ,   0x24ff},  // Memory Model 12
};

const WORD NUMofMemModels = sizeof(sMemModel)/sizeof(sMemModel[0]);

// Memory Definition.
static WORD MemoryConfig[] = 
{
   MemModNo,   // Device unknown
   MemMod_2P,  // MSP430F1121   has 4k     bytes main memory starting at 0xf000
   MemMod_2,   // MSP430F1121D  has 4k     bytes main memory starting at 0xf000
   MemMod_3,   // MSP430F133    has 8k     bytes main memory starting at 0xe000
   MemMod_5,   // MSP430F135    has 16k    bytes main memory starting at 0xc000
   MemMod_7,   // MSP430F147    has 32k    bytes main memory starting at 0x8000
   MemMod_8,   // MSP430F148    has 48k    bytes main memory starting at 0x4000
   MemMod_9,   // MSP430F149    has 59.75k bytes main memory starting at 0x1100
   MemMod_3,   // MSP430F413P   has 8k     bytes main memory starting at 0xe000
   MemMod_3,   // MSP430F413C   has 8k     bytes main memory starting at 0xe000
   MemMod_2,   // MSP430F1121A  has 4k     bytes main memory starting at 0xf000
   MemMod_3,   // MSP430F123    has 8k     bytes main memory starting at 0xe000
   MemMod_7,   // MSP430F437    has 32k    bytes main memory starting at 0x8000
   MemMod_9,   // MSP430F449    has 59.75k bytes main memory starting at 0x1100
   MemMod_3,   // MSP430F1232   has 8k     bytes main memory starting at 0xe000
   MemMod_3,   // MSP430F1132   has 8k     bytes main memory starting at 0xe000
   MemMod_3,   // MSP430F1232C  has 8k     bytes main memory starting at 0xe000
   MemMod_9,   // PMS430F169    has 59.75k bytes main memory starting at 0x1100
   MemMod_3,   // MSP430FE423   has 8k     bytes main memory starting at 0xe000
   MemMod_5,   // MSP430FE425   has 16k    bytes main memory starting at 0xc000
   MemMod_7,   // MSP430FE427   has 32k    bytes main memory starting at 0x8000
   MemMod_3,   // MSP430F423    has 8k     bytes main memory starting at 0xe000
   MemMod_5,   // MSP430F425    has 16k    bytes main memory starting at 0xc000
   MemMod_7,   // MSP430F427    has 32k    bytes main memory starting at 0x8000
   MemMod_7,   // MSP430FW427   has 32k    bytes main memory starting at 0x8000
   MemMod_5,   // MSP430F155    has 16k    bytes main memory starting at 0x1100
   MemMod_6,   // MSP430F156    has 24k    bytes main memory starting at 0x1100
   MemMod_7,   // MSP430F157    has 32k    bytes main memory starting at 0x1100
   MemMod_7,   // MSP430F167    has 32k    bytes main memory starting at 0x1100
   MemMod_8,   // MSP430F168    has 48k    bytes main memory starting at 0x1100
   MemMod_9,   // MSP430F169    has 59.75k bytes main memory starting at 0x1100
   MemMod_10,  // MSP430F1610   has 32k    bytes main memory starting at 0x8000 +  5k RAM
   MemMod_11,  // MSP430F1611   has 48k    bytes main memory starting at 0x4000 + 10k RAM
   MemMod_12,  // MSP430F1612   has 55k    bytes main memory starting at 0x2500 +  5k RAM
   MemMod_9,   // MSP430FG439   has 59.75k bytes main memory starting at 0x1100
   0x0000,     // End of devices
};




struct tsDebugInfo {
    WORD nBreakpoints;
    WORD emulation;
    WORD clockControl;
   };

const struct tsDebugInfo sDebugInfo[] = 
{   //Hardware breakpoints
        // Emulation logic.
                     // Clock control.
    { 2,   EMEX_NONE,   GCC_NONE     },  // Device unknown
    { 2,   EMEX_NONE,   GCC_NONE     },  // MSP430F1121
    { 2,   EMEX_LOW,    GCC_NONE     },  // MSP430F1121D
    { 3,   EMEX_MEDIUM, GCC_NONE     },  // MSP430F133
    { 3,   EMEX_MEDIUM, GCC_NONE     },  // MSP430F135
    { 3,   EMEX_MEDIUM, GCC_NONE     },  // MSP430F147
    { 3,   EMEX_MEDIUM, GCC_NONE     },  // MSP430F148
    { 3,   EMEX_MEDIUM, GCC_NONE     },  // MSP430F149
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430F413P
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430F413C
    { 2,   EMEX_LOW,    GCC_NONE     },  // MSP430F1121A
    { 2,   EMEX_LOW,    GCC_NONE     },  // MSP430F123
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F437
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F449
    { 2,   EMEX_LOW,    GCC_NONE     },  // MSP430F1232
    { 2,   EMEX_LOW,    GCC_NONE     },  // MSP430F1132
    { 2,   EMEX_LOW,    GCC_NONE     },  // MSP430F1232C
    { 3,   EMEX_MEDIUM, GCC_NONE     },  // PMS430F169
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430FE423
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430FE425
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430FE427
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430F423
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430F425
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430F427
    { 2,   EMEX_LOW,    GCC_STANDARD },  // MSP430FW427
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F155
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F156
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F157
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F167
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F168
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F169
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F1610
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F1611
    { 8,   EMEX_HIGH,   GCC_EXTENDED },  // MSP430F1612
    { 2,   EMEX_LOW,    GCC_EXTENDED },  // MSP430FG439
    { 0,   EMEX_NONE,   GCC_NONE     },  // End of devices
};




struct tsVoltages {
    WORD vccMinOp;
    WORD vccMaxOp;
    WORD vccMinSecure;
    WORD vppMinSecure;
    WORD vppMaxSecure;
   };

const struct tsVoltages sVoltages[] = 
{ // Vcc minimum during operation [mVolts].
  // |       Vcc maximum during operation [mVolts].
  // |       |         Vcc minimum during secure operation [mVolts]. 0 if no fuse.
  // |       |         |      Vpp minimum during secure operation [mVolts]. 0 if no fuse.
  // |       |         |      |      Vpp maximum during secure operation [mVolts]. 0 if no fuse.
    { 1800,  3600,     0,     0,     0 },  // Device unknown
    { 1800,  3600,     0,     0,     0 },  // MSP430F1121
    { 1800,  3600,     0,     0,     0 },  // MSP430F1121D
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F133
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F135
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F147
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F148
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F149
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F413P
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F413C
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1121A
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F123
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F437
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F449
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1232
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1132
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1232C
    { 1800,  3600,  2500,  6000,  7000 },  // PMS430F169
    { 2700,  3600,  2700,  6000,  7000 },  // MSP430FE423
    { 2700,  3600,  2700,  6000,  7000 },  // MSP430FE425
    { 2700,  3600,  2700,  6000,  7000 },  // MSP430FE427
    { 2700,  3600,  2700,  6000,  7000 },  // MSP430F423
    { 2700,  3600,  2700,  6000,  7000 },  // MSP430F425
    { 2700,  3600,  2700,  6000,  7000 },  // MSP430F427
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430FW427
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F155
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F156
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F157
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F167
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F168
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F169
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1610
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1611
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430F1612
    { 1800,  3600,  2500,  6000,  7000 },  // MSP430FG439
    {    0,     0,     0,     0,     0 },  // End of devices
};




struct tsDeviceFeatures {
    BYTE hasTestVpp;
    BYTE hasDCOPlus;
    BYTE hasI2C;
    BYTE lcdStart;
    BYTE lcdEnd;
	BYTE hasMarginalReadFlash;
	};

const struct tsDeviceFeatures sDeviceFeatures[] = 
{//   TEST/VPP [TRUE] (or TDI/VPP [FALSE]).
 //   |      Clock system.
 //   |      |      Device has I2C Module.
 //   |      |      |      LCD start address. Note: An LCD start address of zero indicates that no LCD is present.
 //   |      |      |      |       LCD end address. Note: An LCD end address of zero indicates that no LCD is present.
 //   |      |      |      |       |       Flash support Marginal Read test
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // Device unknown
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F1121
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F1121D
    { FALSE, FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F133
    { FALSE, FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F135
    { FALSE, FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F147
    { FALSE, FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F148
    { FALSE, FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F149
    { FALSE, TRUE,  FALSE, 0x0090, 0x009c, FALSE }, // MSP430F413P
    { FALSE, TRUE,  FALSE, 0x0090, 0x009c, FALSE }, // MSP430F413C
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F1121A
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F123
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430F437
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430F449
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F1232
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F1132
    { TRUE,  FALSE, FALSE, 0x0000, 0x0000, FALSE }, // MSP430F1232C
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // PMS430F169
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430FE423
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430FE425
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430FE427
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430F423
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430F425
    { FALSE, TRUE,  FALSE, 0x0090, 0x00a4, FALSE }, // MSP430F427
    { FALSE, TRUE,  FALSE, 0x0090, 0x009c, FALSE }, // MSP430FW427
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F155
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F156
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F157
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F167
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F168
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F169
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F1610
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F1611
    { FALSE, FALSE, TRUE,  0x0000, 0x0000, FALSE }, // MSP430F1612
    { FALSE, TRUE,  FALSE, 0x0090, 0x009c, FALSE }, // MSP430FG439
    { FALSE, FALSE, FALSE, 0x0000, 0x0000, FALSE }, // End of devices
};




struct tsDeviceBugs {
    WORD hasTMRBug;
    WORD hasJTAGBug;
    WORD hasDTCBug;
    WORD hasForceSyncBug;
    WORD hasForceInstrLoadBug;
   };

const struct tsDeviceBugs sDeviceBugs[] = 
{//   TMR Bug.
 //   |      JTAG Bug.
 //   |      |      DTC Bug.
 //   |      |      |      Force Synchronization Bug.
 //   |      |      |      |      Force Instruction Load Bug.
    { TRUE,  TRUE,  FALSE, TRUE,  FALSE }, // Device unknown
    { TRUE,  FALSE, FALSE, FALSE, FALSE }, // MSP430F1121
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1121D
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F133
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F135
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F147
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F148
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F149
    { FALSE, TRUE,  FALSE, FALSE, TRUE  }, // MSP430F413P
    { FALSE, TRUE,  FALSE, FALSE, TRUE  }, // MSP430F413C
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1121A
    { FALSE, TRUE,  FALSE, FALSE, FALSE }, // MSP430F123
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430F437
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430F449
    { FALSE, FALSE, TRUE,  FALSE, FALSE }, // MSP430F1232
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1132
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1232C
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // PMS430F169
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430FE423
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430FE425
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430FE427
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430F423
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430F425
    { FALSE, FALSE, FALSE, TRUE,  FALSE }, // MSP430F427
    { FALSE, FALSE, FALSE, FALSE, TRUE  }, // MSP430FW427
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F155
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F156
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F157
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F167
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F168
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F169
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1610
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1611
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F1612
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // MSP430F439
    { FALSE, FALSE, FALSE, FALSE, FALSE }, // End of devices

};



// Special Function Register Masks.
static WORD sfrMasks[][NSFRSMASKED] =
{
   //0x0   0x1   0x2   0x3   0x4   0x5
   //IE1   IE2  IFG1  IFG2   ME1   ME2
   {0xff, 0xff, 0xff, 0xff, 0xff, 0xff}, // Device unknown
   {0x33, 0xff, 0x13, 0xff, 0xff, 0xff}, // MSP430F1121
   {0x33, 0xff, 0x13, 0xff, 0xff, 0xff}, // MSP430F1121D
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F133
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F135
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F147
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F148
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F149
   {0x33, 0x80, 0x13, 0x80, 0xff, 0xff}, // MSP430F413P
   {0x33, 0x80, 0x13, 0x80, 0xff, 0xff}, // MSP430F413C
   {0x33, 0xff, 0x13, 0xff, 0xff, 0xff}, // MSP430F1121A
   {0x33, 0x03, 0x13, 0x03, 0xff, 0x03}, // MSP430F123
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430F437
   {0xf3, 0xb0, 0xd3, 0xb0, 0xc0, 0x30}, // MSP430F449
   {0x33, 0x03, 0x13, 0x03, 0xff, 0x03}, // MSP430F1232
   {0x33, 0xff, 0x13, 0xff, 0xff, 0xff}, // MSP430F1132
   {0x33, 0x03, 0x13, 0x03, 0xff, 0x03}, // MSP430F1232C
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // PMS430F169
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430FE423
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430FE425
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430FE427
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430F423
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430F425
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430F427
   {0x33, 0x80, 0x13, 0x80, 0xff, 0xff}, // MSP430FW427
   {0xf3, 0xff, 0xd3, 0xff, 0xc0, 0xff}, // MSP430F155
   {0xf3, 0xff, 0xd3, 0xff, 0xc0, 0xff}, // MSP430F156
   {0xf3, 0xff, 0xd3, 0xff, 0xc0, 0xff}, // MSP430F157
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F167
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F168
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F169
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F1610
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F1611
   {0xf3, 0x30, 0xd3, 0x30, 0xc0, 0x30}, // MSP430F1612
   {0xf3, 0x80, 0xd3, 0x80, 0xc0, 0xff}, // MSP430FG439
   {0xff, 0xff, 0xff, 0xff, 0xff, 0xff}, // End of devices
};

// (Data access) Functions. ---------------------------------------------------


#ifdef _DEBUG
void error_printf(char * Err)
{
    printf(Err);
}
#endif


char* deviceString(void)
{
   return (sdeviceID[deviceId].IDstring);
}

WORD deviceMainStart(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceMainStart");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].mainStart);
}

ULONG deviceMainEnd(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceMainEnd");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].mainEnd);
}

WORD deviceInfoStart(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceInfoStart");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].infoStart);
}

WORD deviceInfoEnd(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceInfoEnd");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].infoEnd);
}

WORD deviceRamStart(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceRamStart");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].ramStart);
}


WORD deviceRamEnd(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceRamEnd");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].ramEnd);
}

WORD deviceRam2Start(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceRamStart");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].ram2Start);
}


WORD deviceRam2End(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(MemoryConfig)/sizeof(MemoryConfig[0])){error_printf("Error: deviceRamEnd");};
#endif
   return (sMemModel[MemoryConfig[deviceTypeId]].ram2End);
}

WORD deviceNBreakpoints(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDebugInfo)/sizeof(sDebugInfo[0])){error_printf("Error: sDebugInfo");};
#endif
   return (sDebugInfo[deviceTypeId].nBreakpoints);

}

WORD deviceEmulation(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDebugInfo)/sizeof(sDebugInfo[0])){error_printf("Error: sDebugInfo");};
#endif
   return (sDebugInfo[deviceTypeId].emulation);
}

WORD deviceClockControl(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDebugInfo)/sizeof(sDebugInfo[0])){error_printf("Error: sDebugInfo");};
#endif
   return (sDebugInfo[deviceTypeId].clockControl);
}

WORD deviceLcdStart(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDeviceFeatures)/sizeof(sDeviceFeatures[0])){error_printf("Error: sDeviceFeatures");};
#endif
   return (sDeviceFeatures[deviceTypeId].lcdStart);
}

WORD deviceLcdEnd(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDeviceFeatures)/sizeof(sDeviceFeatures[0])){error_printf("Error: sDeviceFeatures");};
#endif
   return (sDeviceFeatures[deviceTypeId].lcdEnd);
}

WORD deviceVccMinOp(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sVoltages)/sizeof(sVoltages[0])){error_printf("Error: sVoltages");};
#endif
   return (sVoltages[deviceTypeId].vccMinOp);
}

WORD deviceVccMaxOp(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sVoltages)/sizeof(sVoltages[0])){error_printf("Error: sVoltages");};
#endif
   return (sVoltages[deviceTypeId].vccMaxOp);
}

WORD deviceVccMinSecure(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
//    while ((DT_END +1) != sizeof(vccMinSecure)/sizeof(vccMinSecure[0])){error_printf("Error: deviceVccMinSecure");};
    while ((DT_END +1) != sizeof(sVoltages)/sizeof(sVoltages[0])){error_printf("Error: sVoltages");};
#endif
//   return (vccMinSecure[deviceTypeId]);
   return (sVoltages[deviceTypeId].vccMinSecure);
}

WORD deviceVppMinSecure(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
//    while ((DT_END +1) != sizeof(vppMinSecure)/sizeof(vppMinSecure[0])){error_printf("Error: deviceVppMinSecure");};
    while ((DT_END +1) != sizeof(sVoltages)/sizeof(sVoltages[0])){error_printf("Error: sVoltages");};
#endif
//   return (vppMinSecure[deviceTypeId]);
   return (sVoltages[deviceTypeId].vppMinSecure);
}

WORD deviceVppMaxSecure(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
//    while ((DT_END +1) != sizeof(vppMaxSecure)/sizeof(vppMaxSecure[0])){error_printf("Error: deviceVppMaxSecure");};
    while ((DT_END +1) != sizeof(sVoltages)/sizeof(sVoltages[0])){error_printf("Error: sVoltages");};
#endif
//   return (vppMaxSecure[deviceTypeId]);
   return (sVoltages[deviceTypeId].vppMaxSecure);
}

WORD deviceHasTestVpp(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDeviceFeatures)/sizeof(sDeviceFeatures[0])){error_printf("Error: sDeviceFeatures");};
#endif
   return (sDeviceFeatures[deviceTypeId].hasTestVpp);
}

WORD deviceHasDCOPlus(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDeviceFeatures)/sizeof(sDeviceFeatures[0])){error_printf("Error: sDeviceFeatures");};
#endif
   return (sDeviceFeatures[deviceTypeId].hasDCOPlus);
}

WORD deviceHasI2C(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDeviceFeatures)/sizeof(sDeviceFeatures[0])){error_printf("Error: sDeviceFeatures");};
#endif
   return (sDeviceFeatures[deviceTypeId].hasI2C);
}

WORD deviceHasMarginalReadFlash(void) 
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sDeviceFeatures)/sizeof(sDeviceFeatures[0])){error_printf("Error: sDeviceFeatures");};
#endif
   return ((WORD)sDeviceFeatures[deviceTypeId].hasMarginalReadFlash);
}

WORD deviceHasTMRBug(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != (sizeof(sDeviceBugs)/sizeof(sDeviceBugs[0]))){error_printf("Error: sDeviceBugs");};
#endif
   return (sDeviceBugs[deviceTypeId].hasTMRBug);
}

WORD deviceHasJTAGBug(void)
{
#ifdef _DEBUG
    // Check if table has correct size

    while ((DT_END +1) != (sizeof(sDeviceBugs)/sizeof(sDeviceBugs[0]))){error_printf("Error: sDeviceBugs");};
#endif
   return (sDeviceBugs[deviceTypeId].hasJTAGBug);
}

WORD deviceHasDTCBug(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != (sizeof(sDeviceBugs)/sizeof(sDeviceBugs[0]))){error_printf("Error: sDeviceBugs");};
#endif
   return (sDeviceBugs[deviceTypeId].hasDTCBug);
}

WORD deviceHasForceSyncBug(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != (sizeof(sDeviceBugs)/sizeof(sDeviceBugs[0]))){error_printf("Error: sDeviceBugs");};
#endif
   return (sDeviceBugs[deviceTypeId].hasForceSyncBug);
}

WORD deviceHasForceInstrLoadBug(void)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != (sizeof(sDeviceBugs)/sizeof(sDeviceBugs[0]))){error_printf("Error: sDeviceBugs");};
#endif
   return (sDeviceBugs[deviceTypeId].hasForceInstrLoadBug);
}

WORD deviceSfrMasks(int mask)
{
#ifdef _DEBUG
    // Check if table has correct size
    while ((DT_END +1) != sizeof(sfrMasks)/sizeof(sfrMasks[0])){ error_printf("Error: deviceSfrMasks");};
#endif
   return (sfrMasks[deviceTypeId][mask]);
}
