// Device parameters.
//
// Version 1.0
//
// Texas Instruments, 2002

#ifndef _DEVICES_H_
#define _DEVICES_H_

// #defines. ------------------------------------------------------------------

// Devices Types. One of the following enumerations is returned in device.id
enum 
{
    DT_UNKNOWN,
    DT_MSP430F11x1,  // Version A silicon.
    DT_MSP430F11x1D, // Version D silicon (changed IM location and EMEX BP logic)
    DT_MSP430F133,
    DT_MSP430F135,
    DT_MSP430F147,
    DT_MSP430F148,
    DT_MSP430F149,
    DT_MSP430F413P,
    DT_MSP430F41xC,
    DT_MSP430F11x1A,
    DT_MSP430F12x,
    DT_MSP430F43x,   // 80 + 100 Pin devices
    DT_MSP430F44x,   // F44x (100 Pin devices)
    DT_MSP430F12x2,
    DT_MSP430F11x2,
    DT_MSP430F12x2C,
    DT_PMS430F16x,   // Preliminary  Version PMS430F169
    DT_MSP430FE423,
    DT_MSP430FE425,
    DT_MSP430FE427,
    DT_MSP430F423,   // MSP430F423 with SD16
    DT_MSP430F425,   // MSP430F425 with SD16
    DT_MSP430F427,   // MSP430F427 with SD16
    DT_MSP430FW42x,
    DT_MSP430F155,   // MSP430F155
    DT_MSP430F156,   // MSP430F156
    DT_MSP430F157,   // MSP430F157
    DT_MSP430F167,   // MSP430F167
    DT_MSP430F168,   // MSP430F168
    DT_MSP430F169,   // MSP430F169
    DT_MSP430F1610,  // MSP430F1610
    DT_MSP430F1611,  // MSP430F1611
    DT_MSP430F1612,  // MSP430F1612
    DT_MSP430FG43x,  // MSP430FG439
    DT_END ,    // This enumeration must be the last device.
};


// Number of Special Function Registers masks (0x0000-0x0005)
#define NSFRSMASKED 6

// Number of DeviceID Parameter
#define NDEVICEIDPARAMETER 9

/**/
extern const WORD NUMofDevices;
struct tsdeviceID {
	 short deviceIDParameter[NDEVICEIDPARAMETER];
	 WORD  deviceTypeID;
	 char* IDstring;
	};
extern const struct tsdeviceID sdeviceID[];
extern int deviceTypeId;	    // Device identification for device Table

/**/

// Functions. -----------------------------------------------------------------

char* deviceString(void);		// Device string.
WORD  deviceMainStart(void);	// Main memory start address.
ULONG deviceMainEnd(void); 		// Main memory end address. 
WORD  deviceInfoStart(void);	// Information memory start address.
WORD  deviceInfoEnd(void);      // Information memory end address. 
WORD  deviceRamStart(void);		// Ram memory Start address.
WORD  deviceRamEnd(void);		// Ram memory end address.
WORD  deviceRam2Start(void);	// Ram2 memory Start address.
WORD  deviceRam2End(void);		// Ram2 memory end address.
WORD  deviceNBreakpoints(void);	// Hardware breakpoints.
WORD  deviceEmulation(void);	// Emulation logic.
WORD  deviceClockControl(void);	// Clock control.
WORD  deviceLcdStart(void);		// LCD start address.
WORD  deviceLcdEnd(void);		// LCD end address.
WORD  deviceVccMinOp(void);		// Vcc minimum during operation [mVolts].
WORD  deviceVccMaxOp(void);		// Vcc maximum during operation [mVolts].
WORD  deviceVccMinSecure(void);	// Vcc minimum during secure operation [mVolts].
WORD  deviceVppMinSecure(void);	// Vpp minimum during secure operation [mVolts].
WORD  deviceVppMaxSecure(void);	// Vpp maximum during secure operation [mVolts].
WORD  deviceHasTestVpp(void);	// TEST/VPP [TRUE] (or TDI/VPP [FALSE]).
WORD  deviceHasDCOPlus(void);	// Clock system.
WORD  deviceHasI2C(void);       // Device with I2C Module 
WORD  deviceHasMarginalReadFlash(void); // Device as Marginal Read Flash
WORD  deviceHasTMRBug(void);		// TMR Bug.
WORD  deviceHasJTAGBug(void);	// JTAG Bug.
WORD  deviceHasDTCBug(void);	// DTC Bug.
WORD  deviceHasForceSyncBug(void); // Force Synchronization Bug.
WORD  deviceHasForceInstrLoadBug(void); // Force Instruction Load Bug.
WORD  deviceSfrMasks(int mask);	// Special Function Register Masks.

#endif // _DEVICES_H_
