/*========================================================================*\
|                                                                          |
| MSP430.h                                                                 |
|                                                                          |
| This file contains the JTAG instruction and control bit definitions for  |
| the MSP430                                                               |
|--------------------------------------------------------------------------|
| Project:              MSP430 JTAG interfcae                              |
| Developed using:      MS Visual C++ 5.0                                  |
|--------------------------------------------------------------------------|
| Version:              1.1.5                                              |
| Initial Version:      20 / 07 / 02                                       |
| Last Change:          01 / 22 / 04                                       |
|--------------------------------------------------------------------------|
| Version history:                                                         |
| Version: 1.1.5                                                           |
| 12/02/2003 UPSF Added mirrowed JTAG-instr. table for uC execution        | 
| 12/02/2003 UPSF Changed JMP_$ to JMP_D                                   | 
| 22/02/2003 UPSF Renamed IEx, I2C and ADC10 Register definitions          |
|                                                                          |
|--------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments                                       |
\*========================================================================*/

/*------------------------------------------------------------------------*\
| Remarks:                                                                 |
|                                                                          |
\*------------------------------------------------------------------------*/

#ifndef _JTAG_defs_H_
#define _JTAG_defs_H_

// #defines. ------------------------------------------------------------------
#define FUSECHECK_DELAY         50

#define JTAGVERSION			    0x89

#ifndef uController  // defines for MSVCC
// Instructions to access the Emex registers
#define	IR_EMEX_DATA_EXCHANGE	0x09
#define	IR_EMEX_READ_TRIGGER	0x0A
#define	IR_EMEX_READ_CONTROL	0x0B
#define	IR_EMEX_WRITE_CONTROL	0x0C

// Instructions for the address register
#define IR_ADDR_HIGH_BYTE		0x81
#define IR_ADDR_LOW_BYTE		0x82
#define IR_ADDR_16BIT			0x83
#define IR_ADDR_CAPTURE			0x84
#define IR_DATA_TO_ADDR			0x85
#define IR_CAPTURE_CPU_REG		0x86

// Instructions for the data register
#define IR_DATA_16BIT			0x41
#define IR_DATA_CAPTURE			0x42
#define IR_DATA_QUICK			0x43
#define IR_DATA_PSA	            0x44
#define IR_DATA_16BIT_OPT		0x45
#define IR_SHIFT_OUT_PSA		0x46
#define IR_DTA			        0x47

// Instructions for the breakpoint logic
#define IR_BP_CNTL_16BIT		0x09
#define IR_BP_CNTL_CAPTURE		0x0A
#define IR_BP1_16BIT			0x0B
#define IR_BP1_CAPTURE			0x0C
#define IR_BP2_16BIT			0x0D
#define IR_BP2_CAPTURE			0x0E

// Instructions for the FLASH register
#define IR_FLASH_16BIT_UPDATE	0x19
#define IR_FLASH_CAPTURE		0x1A
#define IR_FLASH_16BIT_IN		0x1B
#define IR_FLASH_UPDATE			0x1C
// Bits of the FLASH register
#define FLASH_SESEL1			0x0080
#define FLASH_TMR			    0x0800

// Instructions for the control signal register
#define IR_CNTRL_SIG_HIGH_BYTE	0x11
#define IR_CNTRL_SIG_LOW_BYTE	0x12
#define IR_CNTRL_SIG_16BIT		0x13
#define IR_CNTRL_SIG_CAPTURE	0x14
#define IR_CNTRL_SIG_RELEASE	0x15
// Bits of the control signal register
#define CNTRL_SIG_READ			0x0001
#define CNTRL_SIG_CPU_HALT		0x0002
#define CNTRL_SIG_INTR_REQ		0x0004
#define CNTRL_SIG_HALT_JTAG		0x0008
#define CNTRL_SIG_BYTE			0x0010
#define CNTRL_SIG_CPU_OFF		0x0020
#define CNTRL_SIG_MCLKON		0x0040
#define CNTRL_SIG_INSTRLOAD		0x0080
#define CNTRL_SIG_TMODE			0x0100
#define CNTRL_SIG_TCE			0x0200
#define CNTRL_SIG_TCE1			0x0400
#define CNTRL_SIG_PUC			0x0800
#define CNTRL_SIG_CPU			0x1000
#define CNTRL_SIG_TAGFUNCSAT	0x2000
#define	CNTRL_SIG_SWITCH		0x4000
#define CNTRL_SIG_STOP_SEL		0x8000

// Instructions for the fuse control
#define IR_CNTRL			    0x21
#define IR_PREPARE_BLOW			0x22
#define IR_EX_BLOW			    0x24

// Instructions for the configuration fuse register
#define IR_CONFIG_FUSES			0x29

// Instructions for the controll of the Embedded Signal Processing Cell
#define IR_DUAL_8BIT			0x31
#define IR_DUAL_CAPTURE			0x32
#define IR_SELECT_MAIN			0x33
#define IR_SELECT_ESP			0x34

// Bypass instruction
#define IR_BYPASS			    0xFF

#else // uController part
// Instructions to access the Emex registers
#define	IR_EMEX_DATA_EXCHANGE	0x90  // 09
#define	IR_EMEX_READ_TRIGGER	0x50  // 0A
#define	IR_EMEX_READ_CONTROL	0xD0  // 0B
#define	IR_EMEX_WRITE_CONTROL	0x30  // 0C

// Instructions for the address register
#define IR_ADDR_HIGH_BYTE		0x81  // 81
#define IR_ADDR_LOW_BYTE		0x41  // 82
#define IR_ADDR_16BIT			0xC1  // 83
#define IR_ADDR_CAPTURE			0x21  // 84
#define IR_DATA_TO_ADDR			0xA1  // 85
#define IR_CAPTURE_CPU_REG		0x61  // 86

// Instructions for the data register
#define IR_DATA_16BIT			0x82  // 41
#define IR_DATA_CAPTURE			0x42  // 42
#define IR_DATA_QUICK			0xC2  // 43
#define IR_DATA_PSA		    	0x22  // 44
#define IR_DATA_16BIT_OPT		0xA2  // 45
#define IR_SHIFT_OUT_PSA		0x62  // 46
#define IR_DTA				    0xE2  // 47

// Instructions for the breakpoint logic
#define IR_BP_CNTL_16BIT		0x90  // 09
#define IR_BP_CNTL_CAPTURE		0x50  // 0A
#define IR_BP1_16BIT			0xD0  // 0B
#define IR_BP1_CAPTURE			0x30  // 0C
#define IR_BP2_16BIT			0xB0  // 0D
#define IR_BP2_CAPTURE			0x70  // 0E

// Instructions for the FLASH register
#define IR_FLASH_16BIT_UPDATE	0x98  // 19
#define IR_FLASH_CAPTURE		0x58  // 1A
#define IR_FLASH_16BIT_IN		0xD8  // 1B
#define IR_FLASH_UPDATE			0x38  // 1C
// Bits of the FLASH register
#define FLASH_SESEL1			0x0080
#define FLASH_TMR			    0x0800

// Instructions for the control signal register
#define IR_CNTRL_SIG_HIGH_BYTE	0x88  // 11
#define IR_CNTRL_SIG_LOW_BYTE	0x48  // 12
#define IR_CNTRL_SIG_16BIT		0xC8  // 13
#define IR_CNTRL_SIG_CAPTURE	0x28  // 14
#define IR_CNTRL_SIG_RELEASE	0xA8  // 15
// Bits of the control signal register
#define CNTRL_SIG_READ			0x0001
#define CNTRL_SIG_CPU_HALT		0x0002
#define CNTRL_SIG_INTR_REQ		0x0004
#define CNTRL_SIG_HALT_JTAG		0x0008
#define CNTRL_SIG_BYTE			0x0010
#define CNTRL_SIG_CPU_OFF		0x0020
#define CNTRL_SIG_MCLKON		0x0040
#define CNTRL_SIG_INSTRLOAD		0x0080
#define CNTRL_SIG_TMODE			0x0100
#define CNTRL_SIG_TCE			0x0200
#define CNTRL_SIG_TCE1			0x0400
#define CNTRL_SIG_PUC			0x0800
#define CNTRL_SIG_CPU			0x1000
#define CNTRL_SIG_TAGFUNCSAT	0x2000
#define	CNTRL_SIG_SWITCH		0x4000
#define CNTRL_SIG_STOP_SEL		0x8000

// Instructions for the fuse control
#define IR_CNTRL			    0x84  // 21
#define IR_PREPARE_BLOW			0x44  // 22
#define IR_EX_BLOW			    0x24  // 24

// Instructions for the configuration fuse register
#define IR_CONFIG_FUSES			0x94  // 29

// Instructions for the controll of the Embedded Signal Processing Cell
#define IR_DUAL_8BIT			0x8C  // 31
#define IR_DUAL_CAPTURE			0x4C  // 32
#define IR_SELECT_MAIN			0xCC  // 33
#define IR_SELECT_ESP			0x2C  // 34

// Bypass instruction
#define IR_BYPASS			    0xFF


#endif


// Registers in the EMEX logic
#define MX_WRITE			0	   // Write offset
#define MX_READ				1	   // Read offset

// Breakpoint block
// EMEX address = BP number * Block size + Register offset + R/W offset
// Note: In Volker's new EEM documentation, this block is called the Memory Bus Trigger
#define MX_BLKSIZE		    	8	   // Block size
#define MX_BP			    	0x0000 // Breakpoint value offset
#define MX_CNTRL		    	0x0002 // Control offset
#define MX_MASK			    	0x0004 // Mask offset
#define MX_COMB			    	0x0006 // Combination offset

// Control block
#define	MX_CPUSTOP		    	0x0080
#define	MX_GENCNTRL		    	0x0082
#define MX_GCLKCTRL		    	0x0088
#define MX_MCLKCNTL0	    	0x008A

// Settings of the Breakpoint block Control register
#define	BPCNTRL_MAB			    0x0000
#define	BPCNTRL_MDB			    0x0001
#define	BPCNTRL_RW_DISABLE	    0x0000
#define	BPCNTRL_RW_ENABLE	    0x0020
#define	BPCNTRL_EQ			    0x0000
#define	BPCNTRL_GE			    0x0008
#define	BPCNTRL_LE			    0x0010
#define	BPCNTRL_FREE		    0x0018
#define	BPCNTRL_DMA_DISABLE	    0x0000
#define	BPCNTRL_DMA_ENABLE	    0x0040
// With BPCNTRL_DMA_DISABLE and BPCNTRL_RW_DISABLE
#define	BPCNTRL_IF			    0x0000
#define	BPCNTRL_IFHOLD		    0x0002
#define	BPCNTRL_NIF			    0x0004
#define	BPCNTRL_BOTH		    0x0006
// With BPCNTRL_DMA_DISABLE and BPCNTRL_RW_ENABLE
#define	BPCNTRL_NIF_READ	    0x0000
#define	BPCNTRL_NIF_WRITE	    0x0002
#define	BPCNTRL_READ		    0x0004
#define	BPCNTRL_WRITE		    0x0006
// Special consideration for future data breakpoints and the F12x2:
// - Must handle DMA access (Bit 6 of control register)
// - Read/Write Enable (Bit 5 of control register) is not implemented in the F12x2
// With BPCNTRL_DMA_ENABLE and BPCNTRL_RW_DISABLE
#define	BPCNTRL_NIF_NDMA		0x0000
#define	BPCNTRL_DMA			    0x0002
#define	BPCNTRL_NDMA			0x0004
#define	BPCNTRL_WRITE_NDMA		0x0006
// With BPCNTRL_DMA_ENABLE and BPCNTRL_RW_ENABLE
#define	BPCNTRL_NIF_READ_NDMA	0x0000
#define	BPCNTRL_READ_NDMA		0x0002
#define	BPCNTRL_READ_DMA		0x0004
#define	BPCNTRL_WRITE_DMA		0x0006

// Settings of the Breakpoint block Mask register
#define	BPMASK_WORD			    0x0000
#define BPMASK_HIGHBYTE			0x00FF
#define BPMASK_LOWBYTE			0xFF00
#define BPMASK_DONTCARE			0xFFFF

// MSP430_Debug.h contains definitions for the following:
//  Bits of the Control block General Control register
//  Bits of the Control block General Clock Control register (F41x)
//  Bits of the Control block General Clock Control register (F43x/F44x)

// Bits of the device Status register (R2)
#define STATUS_REG_C			0x0001
#define STATUS_REG_Z			0x0002
#define STATUS_REG_N			0x0004
#define STATUS_REG_GIE			0x0008
#define STATUS_REG_CPUOFF		0x0010
#define STATUS_REG_OSCOFF		0x0020
#define STATUS_REG_SCG0			0x0040
#define STATUS_REG_SCG1			0x0080
#define STATUS_REG_V			0x0100

#define WDT	                    0x120	// Watchdog Timer.
#define WDTPW				    0x5a	// Watchdog Timer password.
#define WDTHOLD				    0x80	// Watchdog Timer hold.

#define FLASHPW				    0xa5	// Flash password.

// MSP430 Instructions.
#define JMP_D				    0x3fff	// JMP $
#define JMP_D_2				    0x3ffe	// JMP $-2
#define JMP_D_4				    0x3ffd	// JMP $-4
#define BIS_B_IMM_1			    0xd3d2	// BIS.B #1,
#define BIC_B_IMM_1			    0xc3d2	// BIC.B #1,
#define MOV_IMM_PC			    0x4030	// MOV #<val>,PC

// MSP430 Address.
#define bIE1				    0x0000
#define bIE2				    0x0001

#define bADC10DTC0			    0x0048
#define wADC10CTL0			    0x01b0
#define wADC10CTL1			    0x01b2

#define bI2CCTL                 0x0070
#define bI2CTCTL                0x0071
#define bI2CDR                  0x0076

#define BYTE_REG_START_ADDR		0x0000
#define WORD_REG_START_ADDR		0x0100
#define LAST_PERIPHERAL_ADDR    0x01ff
//#define DATA_START_ADDR		    0x0200
//#define DATA_END_ADDR			0   x09ff
// ROM address (for use in work-around to RAM-corrupted-during-JTAG-access bug).
#define ROM_ADDR		        0x0c04
//#define FLASH_START_ADDR	        0x1000
//#define FLASH_END_ADDR	        0xffff

#define MAIN_SEGMENT_SIZE		512		// Segments are normally 512 bytes in size.
#define FIRST_60K_SEGMENT_SIZE  256		// However, the first segment of 60K devices is 256 bytes in size.
#define INFO_SEGMENT_SIZE		128		// And Information segments are 128 bytes in size.

#endif /* _JTAG_defs_H_ */
