/**
 * \file
 *  $Author: bpisupat $
 *  $Date: 2007/06/05 16:01:43 $
 *  $Revision: 1.1.1.1 $
*/

#include <stdlib.h>
#include <stdio.h>
#include "arch/thread.h"

/**
 * Currently executing thread
 */

Thread *currentThread;
static struct {
  Thread *hd;
  Thread *tl;
} ready;


/**
 * List of threads that are ready to run
 */



/**
 *  Thread scheduler -- scheduling algorithm is roundrobin
 *
 *  
 *  The thread scheduler should only be called from the
 *  "main" routine.  It returns when no threads are ready.
 *  The intent of this behavior is to allow a processor to
 *  enter low power mode
 *
 * \return Returns when there are no ready threads
 */

void
_threadsched()
{
 
 
  while (1)
    {
      splhi(); // interrupts off

      if (currentThread && (currentThread->state == Running))
	{
	  currentThread->state = Ready;
	  if (ready.tl)
	    {
	      ready.tl->next = currentThread;
	    }
	  else
	    {
	      ready.hd = currentThread;
	    }
	  ready.tl = currentThread;
	  currentThread->next = NULL;
	}
      if (ready.hd)
	{
	  currentThread = ready.hd;
	  ready.hd = ready.hd->next;
	  currentThread->next = NULL;
	  if (ready.hd == NULL)
	    ready.tl = NULL;
	  currentThread->state = Ready;
	}
      else
	{
	  return;
	}

      spllo();
      currentThread->state = Running;
      //      printf("sched restoring %p\n", currentThread);
      // threadBegin();
      currentThread->run();
    }
}

/** 
 *  Make a thread ready to run -- place it on the ready list
 *  and set its state to Ready.
 *
 *  May be called by an interrupt handler to reschedule an
 *  idle thread.
 */

void
_threadready(Thread *t)
{
  int pri = splhi();
  if ((t->state == Ready) || (t->state == Running))
    return;
  t->state = Ready;
 /// printf("sched::setting %c to ready: addr = %p\n",t->name,t);
  if (ready.tl) 
    ready.tl->next = t;
  else
    ready.hd = t;
  ready.tl = t;
  t->next = NULL;
  splx(pri);
}
