#include <string.h>
#include "dat.h"
#include "fns.h"
#include "error.h"

iFid *fidlisthdptr;
iFid *fidlisttlptr;

void initFids() {
  fidlisthdptr = 0;
  fidlisttlptr = 0;
}

void
ifidclone(iFid **nf, iFid *f, ErrorStruct *es)
{
  devtab[f->type]->clone(nf, f);
}

void
ifidclose(iFid *f, ErrorStruct *es)
{
  if(f == 0)
    return;

  if(f->flag&CFREE)
    error(es, "closing closed ifid");

  //  if (c->from)
  //  {
  //    cclose(c->from);
  //    c->from = nil;
  //  }

  /*	       
  if(!waserror()){
    devtab[c->type]->close(c);
    poperror();
  }
  */

  f->flag = CFREE;
  free(f);
}

iFid*
lookupfid(int fid)
{
  iFid *f;
  for (f = fidlisthdptr; f; f = f->next)
    if (f->fidno == fid)
      return f;
  return nil;
}


void isdir(iFid *f, ErrorStruct *es)
{
  if(f->qid.path & DMDIR)
    return;
  error(es, Enotdir);
}

iFid*
newifid(int fidno, ErrorStruct *es)
{
  iFid *f;
  f = (iFid *) malloc(sizeof(iFid));
  // fid ??


  if (f == nil)
    error(es, Enomem);

  f->fidno = fidno;
  f->type = 0;
  f->flag = 0;
  f->dev = 0;
  f->offset = 0;
  f->dri = 0;
  f->next = 0;
  f->prev = 0;
  if (!(fidlisthdptr))
    fidlisthdptr = f;
  if (fidlisttlptr) {
    fidlisttlptr->next = f;
    f->prev = fidlisttlptr;
  }
  fidlisttlptr = f;
    
  
  return f;
}

void
freeifid(iFid *f)
{
  if (f == fidlisthdptr)
    fidlisthdptr = f->next;
  else
    f->prev->next = f->next;
  if (fidlisttlptr == f)
    fidlisttlptr = fidlisttlptr->prev;
  else
  if (f->next)
    f->next->prev = f->prev;

  free(f);
}
    

int
isdotdot(char *p)
{
  return p[0]=='.' && p[1]=='.' && p[2]=='\0';
}


/*
 * Always leaves iFid on the upstream side of a mount point
 *  -- open should cross this 
 */

int walk(iFid **fp, Fcall *rx, Fcall *tx, Channel *cpipe, Channel *cfidpipe, ErrorStruct *es)
{
  char *name;
  int dotdot = 0;
  int rval = 0;
  int len;
  char command = 'k'; //'k' is walk


  isdir(*fp, es);

  name = rx->ns.un.Twcf.name;
  if (name[0] == 0) 
    return 0;

  dotdot = 0;
  if (name[0] == '.' && name[1] == '.' && name[2] == '\0')
    {
      dotdot = 1;
    }

  if (channbsend(cpipe, &rx) == LASTALT)
    return -1; 
  if (channbsend(cfidpipe, fp) == LASTALT)
    return -1; 
  if (channbsend(cpipe, &tx) == LASTALT)
    return -1; 

  return 1; 
}

void
fidopen(iFid** f, int omode, ErrorStruct *es)
{
  //nc = domount(nc);
  devtab[(*f)->type]->open(f, omode, es);
  //  poperror();
}   


