#include <stdio.h>
#include <string.h>

#include "error.h"
#include "treenode.h"
#include "mount.h"
#include "fns.h"
#include "styx.h"
#include "arch/thread.h"

#define STREQ(a,b) !(strcmp(a,b))
#define LOG(x) fprintf(stderr, x);


//#define stderr stdout

ErrorStruct treeerrorstrt;

extern Dev treedev;
TreeNode* filetree;
unsigned int nextPath = 1;

//Callpipe treepipe;
/* channel for passing requests to devtree */
Channel treeChan = CHANNELN(Fcall *, 4);
Channel treefidChan = CHANNELN(iFid *, 4);

//extern Callpipe outpipe;
extern Channel outChan;

Thread treeT;

void treemain();


static
Dirtab* createfile(char* name, unsigned int path, long perm) {
  Dirtab* f = (Dirtab*) malloc(sizeof(Dirtab));
  strcpy(f->name, name);
  f->qid = path;
  f->perm = perm | PDIR;
  return f;
}

unsigned int addFile(char* name, unsigned int parent, long perm) {
  Dirtab* f = createfile(name, nextPath++, perm);
  tree_addchild(tree_findfile(filetree, parent), f);
  
  return f->qid;
}


static void
treeinit(Dev *devptr) {
  Dirtab* rt = createfile(".", 0, 05);
  unsigned int a, b, c, d;
  //  Dirtab* a  = createfile("a", 1, 05);  

  filetree = (TreeNode*) malloc(sizeof(TreeNode));
  
  bzero(filetree, sizeof(TreeNode));
  filetree->filenode = rt;
  filetree->parent = filetree;

  //  tree_addchild(filetree, a);
  //  tree_addchild(filetree, createfile("b", 2, 05));
  //  tree_addchild(filetree, createfile("c", 3, 05));  
  //  tree_addchild(tree_findfile(filetree, a->qid), createfile("e", 5, 05));
  a = addFile("a", 0, 05);
  b = addFile("testdevdir", 0, 05);
  c = addFile("mspdevdir", 0, 05);
  d = addFile("armdevdir", 0, 05);

  treeT.run = treemain;
  setThrdName(treeT,'T');
  _threadready(&treeT);
  //  d = addFile("d", a, 05);
  //  e = addFile("e", a, 05);
  
  //  tree_print(filetree, 0);
}


static void
treeread(iFid *f, void *buf, ushort n, vlong offset, ushort *len)
{
  fprintf(stderr, "treeread: %d\n", f->qid.path & 255);

  int i = 0;
  TreeNode* t = tree_findfile(filetree, f->qid.path & 255);
  TreeNodeList* p = t->children;
  
  // get the appropriate child
  while(i < f->dri) {
    if (p) {
      p = p->next;
      i++;
    } else {
      break;
    }
    
  }

  /* 
   we reach here while doing 'ls', when the client sends progressively larger offsets 
    to get content of whole directory 
  */

  if (!p) {
    *len = 0;
    return;
  }

  devstat(f, (Dir*) buf, p->data->filenode, &treeerrorstrt);
  f->offset += sizeof(Dir);
  f->dri++;
  *len = sizeof(Dir);
  return;
}

static void
treewrite(iFid *f, void *buf, ushort n, vlong offset, ushort *len)
{
  fprintf(stderr, "treewrite: %d\n", f->qid.path & 255);
  error(&treeerrorstrt, Eperm);
}

static void
treeattach(iFid** f, char *spec)
{
  uchar q;
  *f = newifid(-1, &treeerrorstrt);
  (*f)->dev = devno('R');
  (*f)->type = devno('R');
  q = filetree->filenode->qid;
  (*f)->qid.path = (q | DMDIR);
}


static void
treewalk(iFid *f, char *name)
{
  TreeNode* t = tree_findfile(filetree, f->qid.path & 255);
  TreeNodeList* p;

  if (t == nil) 
    error(&treeerrorstrt, "Walk failed: Can't find qid in chan.");

  if (name[0] == '.' && name[1] == 0)
    return;
  if (name[0] == '.' && name[1] == '.' && name[2] == 0)  {

    f->qid.path = t->parent->filenode->qid | DMDIR;
    f->qid.version =  0;

    return;
  }
  
  p = t->children;

  while(p) {
    if (STREQ(name, p->data->filenode->name)) {

      f->qid.path = p->data->filenode->qid + f->type*256;
      f->qid.path |= DMDIR;
      f->qid.version = 0;

      domount(f);
      return;
    } 
    p = p->next;
      
  }

  error(&treeerrorstrt, "Walk failed");  
  
}

void
treestat(iFid *f, Dir *d, ErrorStruct *es )
{
  TreeNode* n = tree_findfile(filetree, f->qid.path & 255);
  fprintf(stderr, "treestat: %s\n", n->filenode->name);
  devstat(f, d, n->filenode, es);
}

// all operations handled by serve get passed an Errorstruct
static void
treeopen(iFid **f, int omode, ErrorStruct *es)
{
  //  fprintf(stderr, "treeopen: %d\n", (*c)->qid.path & 255);

  (*f)->offset = 0;
  (*f)->dri    = 0;
  if(((*f)->qid.path&DMDIR) && omode!=OREAD)
    error(es, Eperm);
  (*f)->mode = omode & (OEXEC);
  (*f)->flag |= COPEN;


}

void treemain() {
  //  Fid *f;
  static Fcall *rx;
  static Fcall *tx;
 
  static iFid *f;
  void *buf;
  u_int16_t count;
  u_int64_t offset;
 
  char *name;

  threadBegin();  

  while(1) {
    // receive call from the call pipe
    if (chanrecv(&treeChan, &rx) < 0)
      threadReturn();
    printf("treethread: recvd 1  request\n");
    if (chanrecv(&treefidChan, &f) < 0)
      threadReturn();
    if (chanrecv(&treeChan, &tx) < 0)
      threadReturn();


    if (!(setjmp(treeerrorstrt.estack))) 
      switch(rx->type){
	
      case Tread:
	count = rx->ns.un.Twrf.count;
	offset = rx->ns.un.Twrf.offset;
	treeread(f, tx->ns.un.Rwrf.rdata + 1, count, offset, &(tx->ns.un.Rwrf.rcount));    
	break;
	
      case Twrite:
	count = rx->ns.un.Twrf.count;
	offset = rx->ns.un.Twrf.offset;
	buf = rx->ns.un.Twrf.data;
	treewrite(f, buf, count, offset, &(tx->ns.un.Rwrf.rcount));
	break;
	
      case Twalk:
	name = rx->ns.un.Twcf.name;
	treewalk(f, name);
	tx->ns.un.qid = f->qid;
	break;
	
      default:
	tx->type = Rerror;
	strcpy(tx->ename, "Invalid operation on treedev");
      }
      else
	{
	  tx->type = Rerror;
	  strcpy(tx->ename, treeerrorstrt.errstr);
	}

    if (chansend(&outChan, &tx) < 0)
      threadReturn();    
  }
  // call the corresponding function
  
  // send the response out of the output pipe
  
  // return the buffer
  threadFinish();

}

void treeclone(iFid **ppc, iFid *pc) {
  devclone(ppc, pc, &treeerrorstrt);
}

void treecreate(iFid* pc, char* pch, int i, ulong ul, ErrorStruct *es) {
  devcreate(pc, pch, i, ul, es);
}

void treeclose(iFid *pc) {
  devclose(pc, &treeerrorstrt);
}

void treeremove(iFid* pc, ErrorStruct *es) {
  printf("treeremove\n");
   devremove(pc, es);
}

void treewstat(iFid* pc, Dir* pd, ErrorStruct *es) {
   devwstat(pc, pd, es);
}

Dev treedev = {	
  'R',
  "tree",
  &treeChan,
  &treefidChan,
  treeinit,
  treeattach,
  treeclone,
  treewalk,
  treestat,
  treeopen,
  treecreate,
  treeclose,
  treeread,
  treewrite,
  treeremove,
  treewstat
};



